"""Boilerplate code for testing purposes"""

from textual.app import App, ComposeResult
from textual_englyph import EnGlyphText


class Test(App):
    """Test console markup styling the englyph text use case"""

    def compose(self) -> ComposeResult:
        yield EnGlyphText("From [red]EnGlyph,", text_size="x-small")
        yield EnGlyphText("Bonjour [dark_orange]Textual!", text_size="small")
        yield EnGlyphText("Olá [bright_yellow]Textual!", text_size="medium")
        yield EnGlyphText("Ciao [green]Textual!", text_size="large")
        yield EnGlyphText("Привiт [cornflower_blue]Textual!", text_size="x-large")
        yield EnGlyphText("Γειά σου [blue1]Textual!", text_size="xx-large")
        yield EnGlyphText("Dobrý deň [violet]Textual!", text_size="xxx-large")

if __name__ == "__main__":
    Test().run()



"""Boilerplate code for testing purposes"""

from textual.app import App, ComposeResult
from textual_englyph import EnGlyph


class Test(App):
    """Test all the pixmap basis and density combinations"""

    phrase_list = ["Hello Textual.", "It's the world!"]
    basis_list = [(2, 4), (2, 3), (2, 2), (1, 2), (1, 1)]
    use_pips = False

    test_widget = EnGlyph("From EnGlyph:")

    def compose(self) -> ComposeResult:
        """main display driver"""
        yield self.test_widget

    def on_mount(self):
        """setup interval time steps at app start"""
        self.set_interval(2.2, self.next_render)

    def next_render(self):
        """step to the next pixmap"""
        this_basis = self.basis_list.pop(0)
        self.basis_list.append(this_basis)

        self.test_widget.update(
            renderable=self.phrase_list[0], basis=this_basis, pips=self.use_pips
        )
        self.set_timer(1, self.rerender)

    def rerender(self):
        """delayed pixmap response for primary interval output"""
        self.test_widget.update(
            renderable=self.phrase_list[1],
            basis=self.basis_list[-1],
            pips=self.use_pips,
        )
        if self.basis_list[-1] == (1, 1):
            self.use_pips = not self.use_pips
            if not self.use_pips:
                self.test_widget.update(
                    renderable="GoodBye!", basis=self.basis_list[-1], pips=self.use_pips
                )
                self.set_timer(1, self.app.exit)


if __name__ == "__main__":
    Test().run()



"""Create seven segment display output for Textual with custom widget EnGlyph"""

from ._englyph_text import EnGlyphText

class EnSevSeg(EnGlyphText):
    """Seven Segement Display"""

    DEFAULT_CSS = """
    EnSevSeg {
        color: red;
        background: #400000;
        border: outer black;
    }
    """
    null_pinput = 0xED00 #Unicode PUA offset for pinput in EmSevSeg.ttf

    _config = {
        "smaller": (-2, "", (0, 0)), # for dynamic update of relative text_size
        "x-small": (1, "", (0, 0)),  # What your terminal normally uses
        "small": (8, "EnSevSeg_8x5.ttf", (2, 4)),
        "larger": (+2, "", (0, 0)), # for dynamic update of relative text_size
    }

    def __init__(
        self,
        *args,
        pinput: int|tuple|list|None = None,
        text_size: str|None = "small",
        **kwargs,
    ):
        super().__init__(
                *args,
                text_size=text_size,
                **kwargs)

    def _pinput2string( self, pinput ):
        pass




"""Create large text output module for Textual with custom widget EnGlyph"""

from rich.console import Console, RenderableType
from rich.text import Text

from textual.strip import Strip

from .englyph import EnGlyph
from .toglyxels import ToGlyxels


class EnGlyphText(EnGlyph):
    """
    A Textual widget to show a variety of large text outputs.
    Process a textual renderable (including Rich.Text)
    Args:
        renderable: Rich renderable or string to display
        text_size:str["medium"], choose size configuration of font
        pips:bool[False], show glyxels in reduced density
        font_name:str[TerminusTTF-4.46.0.ttf], set name/path for font shown in glyxels
        font_size:int[12], set height of font in glyxels, ie. 12pt -> 12gx
        markup:bool[True], Rich Text inline console styling of string
        name:str, Standard Textual Widget argument
        id:str, Standard Textual Widget argument
        classes:str, Standard Textual Widget argument
        disabled:bool, Standard Textual Widget argument
    """

    _config = {
        "smaller": (-2, "", (0, 0)), # for dynamic update of relative text_size
        "xx-small": (0, "", (0, 0)),  # Unicode chars like ᵧ (0x1d67), not implimented
        "x-small": (1, "", (0, 0)),  # What your terminal normally uses
        "small": (8, "miniwi.ttf", (2, 4)),
        "medium": (7, "casio-fx-9860gii.ttf", (2, 4)),
        "large": (7, "casio-fx-9860gii.ttf", (2, 3)),
        "x-large": (12, "TerminusTTF-4.46.0.ttf", (2, 4)),
        "xx-large": (14, "TerminusTTF-4.46.0.ttf", (2, 4)),
        "xxx-large": (18, "TerminusTTF-4.46.0.ttf", (2, 4)),
        "larger": (+2, "", (0, 0)), # for dynamic update of relative text_size
    }

    def __init__(
        self,
        *args,
        text_size: str|None = "x-small",
        font_name: str|None = None,
        font_size: int|None = None,
        basis: tuple|None = None,
        markup: bool = True,
        **kwargs,
    ):
        if font_name is not None:
            basis = basis or (2,4)
        else:
            basis = basis or self._config[text_size][2]
        self._font_size = font_size or self._config[text_size][0]
        self._font_name = font_name or self._config[text_size][1]
        self.markup = markup
        super().__init__(
                *args,
                basis=basis,
                **kwargs)

    def _preprocess(self, renderable: RenderableType | None = None):
        """A stub handler for processing the input _predicate to the renderable"""
        if renderable is not None:
            self._renderable = renderable
            if isinstance(renderable, str):
                if self.markup:
                    self._renderable = Text.from_markup(renderable)
                else:
                    self._renderable = Text(renderable)
        else:
            renderable = self._predicate
        return renderable

    def _process(self) -> None:
        """A stub handler for processing a renderable"""
        self._renderable.stylize_before(self.rich_style)

    def _postprocess(self) -> None:
        """A stub handler to cache a slate (list of strips) from renderable"""
        slate = Console().render_lines(self._renderable, pad=False)
        slate_buf = []
        if self.basis == (0, 0):
            slate_buf = [Strip(strip) for strip in slate]
        else:
            for strip in slate:
                for seg in strip:
                    pane = ToGlyxels.font_pane(
                        seg.text, self._font_name, self._font_size
                    )
                    slate = ToGlyxels.pane2slate(pane, seg.style, self.basis, self.pips)
                    slate_buf = ToGlyxels.slate_join(slate_buf, slate)
        # raise AttributeError( "" )
        self._slate = slate_buf
        return



"""Create large text output module for Textual with custom widget EnGlyph"""

from PIL import ImageOps

from .englyph import EnGlyph
from .toglyxels import ToGlyxels, EnLoad


class EnGlyphImage(EnGlyph):
    """A Textual widget to process a PIL image (or path to) into glyxels.
        Args:
            renderable (PIL Image | path str): The image to be displayed.
            basis (tuple int,int): Glyph pixel (glyxel) partitions in x then y.
            pips (Bool): Are glyxels partition filling or not.
            repeat (int): Number of times an animated image loops.
            Standard Textual Widget Args.
            
        Returns:
            Textual Widget Instance.
    """

    DEFAULT_CSS = """
    EnGlyphImage {
        max-height: 24;
    }
    """

    def __init__(self, *args, repeat: int = 3, **kwargs):
        self._repeats_n = repeat
        super().__init__(*args, **kwargs)

    animate = False

    def _rescale_img(self, img) -> None:
        """Contain the image within height or width keeping aspect ratio or fit image if both"""
        use_width = use_height = False
        try:
            cell_width = self.styles.width.cells
            if cell_width is not None:
                use_width = True
            else:
                cell_width = self.styles.max_width.cells 
        except:
            pass

        try:
            cell_height = self.styles.height.cells
            if cell_height is not None:
                use_height = True
        except:
            pass

        cell_width = cell_width or self.parent.size.width or self.app.size.width
        cell_height = cell_height or self.styles.max_height.cells

        im_size = (self.basis[0] * cell_width, self.basis[1] * cell_height)
        if use_width and use_height:
            im_data = img.resize( im_size )
        else:
            im_data = ImageOps.contain(img, im_size)

        return im_data

    def _update_frame(self, image_frame=None) -> None:
        """accept an image frame to show or move to the next image frame in a sequence"""
        current_frame = self._renderable.tell()
        if image_frame is not None:
            frame = image_frame
        else:
            frame = self._renderable
            if self.animate != 0:
                next_frame = (current_frame + self.animate) % (self._frames_n + 1)
                frame.seek(next_frame)
        self._dblbuff_push(self._rescale_img(frame.convert("RGB")))

    def _dblbuff_init(self) -> None:
        frame = self._rescale_img(self._renderable.convert("RGB"))
        self._slate_cache = ToGlyxels.image2slate(
            frame, basis=self.basis, pips=self.pips
        )
        self._slate = self._slate_cache

    def _dblbuff_push(self, frame) -> None:
        self._slate = self._slate_cache
        self.refresh(layout=True)
        self._slate_cache = ToGlyxels.image2slate(
            frame, basis=self.basis, pips=self.pips
        )

    def _preprocess(self, renderable=None) -> None:
        """A stub init handler to preset "image" properties for glyph processing"""
        if renderable is not None:
            self._renderable = EnLoad( renderable )
        self._frames_n = self._get_frame_count(self._renderable)
        if self._frames_n > 0:
            self.animate = 1
            self._duration_s = self._renderable.info.get("duration", 100) / 1000
        else:
            self.animate = 0
        return renderable

    def _process(self) -> None:
        """A stub on_mount (DOM ready) handler for "image" glyph processing"""
        self._dblbuff_init()
        if self.animate != 0:
            max_frames = self._repeats_n * (self._frames_n + 1)
            self.interval_update = self.set_interval(
                interval=self._duration_s,
                callback=self._update_frame,
                repeat=max_frames,
            )

    def _postprocess(self) -> None:
        """A stub handler to cache a slate (list of strips) from renderable"""
        pass

    def _get_frame_count(self, image):
        frames_n = 0
        image.seek(0)
        while True:
            try:
                image.seek(frames_n + 1)
                frames_n += 1
            except EOFError:
                break
        image.seek(0)
        return frames_n



from textual.app import App, ComposeResult
from textual.containers import VerticalScroll
from textual.widgets import Label
from textual_englyph import EnGlyph


class Test(App):
    DEFAULT_CSS = """
    EnGlyph {
        color: red;
        border: heavy;
    }
    Label {
        color: red;
        border: heavy;
    }
    Grid {
        grid-size: 3 1;
    }
    """

    def compose(self) -> ComposeResult:
        with VerticalScroll():
            yield Label("Hello[blue] Textual[/]!")
            yield EnGlyph("Hello[blue] Textual[/]!")


if __name__ == "__main__":
    Test().run()



"""Boilerplate code for demo"""

from textual.app import App, ComposeResult
from textual.containers import Vertical, Horizontal
from textual.widgets import Header, Footer, Button, TextArea
from textual_englyph import EnGlyph

# pylint: disable=R0801
CONTENT = '''\
from textual.app import App, ComposeResult
from textual_englyph import EnGlyph

class Test(App):
    DEFAULT_CSS = """
    EnGlyph {
        color: green;
        text-style: underline;
        }
    """

    def compose(self) -> ComposeResult:
        yield EnGlyph("EnGlyph [blue]Textual!")

if __name__ == "__main__":
    app = Test()
    app.run()
'''


class MainDemo(App):
    """Test CSS and console markup styling the basic englyph use case"""

    TITLE = "EnGlyph_Demo"
    DEFAULT_CSS = """
    TextArea {
        min-height: 80%;
        width: 57;
        max-width: 57;
    }
    EnGlyph {
        color: green;
        text-style: underline;
        }
    #choice {
        height: 10;
        align: center top;
    }
    """

    code = TextArea(CONTENT)
    code.read_only = True

    def compose(self) -> ComposeResult:
        yield Header()
        yield Footer()
        with Vertical():
            with Horizontal(id="choice"):
                yield Button(str(EnGlyph("PREV")))
                yield EnGlyph("Examples")
                yield Button(str(EnGlyph("NEXT")))
            with Horizontal():
                yield self.code
                yield EnGlyph("EnGlyph [blue]Textual!")


def main_demo():
    """main_demo runner method"""
    app = MainDemo()
    app.run()


if __name__ == "__main__":
    main_demo()



"""Boilerplate code for testing purposes"""

from textual.app import App, ComposeResult
from textual_englyph import EnGlyphText, EnGlyphImage


class Test(App):
    """Test console markup styling the englyph text use case"""
    DEFAULT_CSS = """
    #I {
        max-height: 16;
    }
    .T {
        background: white 50%;
        position: relative;
        offset: 3 -3;
    }
    """

    def compose(self) -> ComposeResult:
        yield EnGlyphImage( "/tmp/logo.jpg", id="I" )
        yield EnGlyphText( " Textual ", classes="T", text_size="medium" )
        #yield EnGlyphText( "Libraries", classes="T", text_size="medium" )


if __name__ == "__main__":
    Test().run(inline=True, inline_no_clear=True)



"""Boilerplate code for testing purposes"""

from textual.app import App, ComposeResult
from textual.containers import Horizontal
from textual_englyph import EnGlyphText, EnSevSeg


class Test(App):
    """Test console markup styling the englyph text use case"""
    DEFAULT_CSS="""
    #clock {
        margin: 1;
        #hours {
            border-right: none;
        }
        #minutes {
            border-left: none;
        }
        #colon {
            color: red;
            background: #400000;
            border-top: outer black;
            border-bottom: outer black;
        }
    }
    """

    def compose(self) -> ComposeResult:
        """Display seven segment digits. Can use Latin basic characters or
        8 bit direct segment control in PUA \uED00 - \uEDFF
        where 1<<0 (LSB) is segement a, 1<<2 is b, ..., 1<<7 is g and 1<<8 is DP
        """
        yield EnSevSeg("Hello Textual")
        yield EnSevSeg("\uEDCF \uED06 4 1 5")
        with Horizontal(id="clock"):
            yield EnSevSeg("12", id="hours")
            yield EnGlyphText(":", text_size="small", id="colon")
            yield EnSevSeg("34", id="minutes")

if __name__ == "__main__":
    Test().run(inline=True, inline_no_clear=True)



"""Boilerplate code for testing purposes"""

# import pyinstrument
from textual.app import App, ComposeResult
from textual_englyph import EnGlyphText, EnGlyphImage


class Test(App):
    """Test the basic englyph image use case"""

    DEFAULT_CSS = """
    Screen {
        background: blue;
    }
    #I {
        height: 16;
    }
    #T {
        background: white 50%;
        position: relative;
        offset: 6 -1;
    }
    """

    def compose(self) -> ComposeResult:
        # yield EnGlyphImage("testing/hopper.jpg", id="I")
        # yield EnGlyphText("'Grace' hopper.jpg", id="T")
        yield EnGlyphImage( "testing/twirl.gif", id="I" )
        yield EnGlyphText( "Coup de Grâce", id="T", text_size="small" )


# uv run testing/image_test.py
if __name__ == "__main__":
    # with pyinstrument.profile():
    Test().run()
# Test().run(inline=True, inline_no_clear=True)



"""Boilerplate code for width testing purposes"""

from textual.app import App, ComposeResult
from textual_englyph import EnGlyph


class Test(App):
    """Test CSS and console markup styling the basic englyph use case"""

    DEFAULT_CSS = """
        #uno {
         color: green;
         max-width: 27;
         }
         #dos {
         color: red;
         }
    """

    second = EnGlyph("Hello [blue]Textual!", basis=(2, 4), id="dos")
    second.styles.width = 15

    def compose(self) -> ComposeResult:
        yield EnGlyph("Hello [blue]Textual!", basis=(2, 4), id="uno")
        yield self.second


if __name__ == "__main__":
    Test().run(inline=True, inline_no_clear=True)



from textual.app import App, ComposeResult
from textual.containers import Grid
from textual.widgets import Label


class Test(App):
    DEFAULT_CSS = """
    Grid {
        grid-size: 2 1;
    }
    Label {
        text-align: right;
    }
    """

    def compose(self) -> ComposeResult:
        yield Grid(Label("Hello"), Label("World!"))


if __name__ == "__main__":
    Test().run()



"""Abstract module class to package up driver routines for EnGlyph"""

# pylint: disable=R0914
# greatly simplifies structure in __init__.py
from typing import List
from importlib import resources

from PIL import Image, ImageFont

from textual.strip import Strip
from rich.segment import Segment
from rich.style import Style
#from rich.traceback import install

#install()
# raise ValueError("My message")

def EnLoad( maybe_path ):
    """
    A function to load file data into memory from a path and return that refernce.
    """
    import io

    if isinstance(maybe_path, str):
        with open(maybe_path, "rb") as fh:
            im_data = fh.read()
            im_buff = io.BytesIO(im_data)
            maybe_path = Image.open(im_buff)
    return maybe_path

class ToGlyxels:
    """Glyph pixels to enable user specified font based string rendering via PIL"""

    @staticmethod
    def font_pane(phrase, font_name, font_size):
        font_asset = resources.files().joinpath("assets", font_name)
        if not font_asset.is_file():
            raise AttributeError( font_asset )
        font = ImageFont.truetype(font_asset, size=font_size)
        _, _, r, b = font.getbbox(phrase)
        # raise AttributeError( r, b )
        mask = list(font.getmask(phrase, mode="1"))
        return (r, b, mask)

    # full infill glyxel(glyph pixel) look up table, columns x rows
    full_glut = [[], ["", "", ""], ["", "", "", "", ""]]
    full_glut[1][1] = " █"
    full_glut[1][2] = " ▀▄█"
    full_glut[2][2] = " ▘▝▀▖▌▞▛▗▚▐▜▄▙▟█"
    full_glut[2][3] = " 🬀🬁🬂🬃🬄🬅🬆🬇🬈🬉🬊🬋🬌🬍🬎🬏🬐🬑🬒🬓▌🬔🬕🬖🬗🬘🬙🬚🬛🬜🬝🬞🬟🬠🬡🬢🬣🬤🬥🬦🬧▐🬨🬩🬪🬫🬬🬭🬮🬯🬰🬱🬲🬳🬴🬵🬶🬷🬸🬹🬺🬻█"
    full_glut[2][4] = (
        " 𜺨𜺫🮂𜴀▘𜴁𜴂𜴃𜴄▝𜴅𜴆𜴇𜴈▀𜴉𜴊𜴋𜴌🯦𜴍𜴎𜴏𜴐𜴑𜴒𜴓𜴔𜴕𜴖𜴗𜴘𜴙𜴚𜴛𜴜𜴝𜴞𜴟🯧𜴠𜴡𜴢𜴣𜴤𜴥𜴦𜴧𜴨𜴩𜴪𜴫𜴬𜴭𜴮𜴯𜴰𜴱𜴲𜴳𜴴𜴵🮅"
        "𜺣𜴶𜴷𜴸𜴹𜴺𜴻𜴼𜴽𜴾𜴿𜵀𜵁𜵂𜵃𜵄▖𜵅𜵆𜵇𜵈▌𜵉𜵊𜵋𜵌▞𜵍𜵎𜵏𜵐▛𜵑𜵒𜵓𜵔𜵕𜵖𜵗𜵘𜵙𜵚𜵛𜵜𜵝𜵞𜵟𜵠𜵡𜵢𜵣𜵤𜵥𜵦𜵧𜵨𜵩𜵪𜵫𜵬𜵭𜵮𜵯𜵰"
        "𜺠𜵱𜵲𜵳𜵴𜵵𜵶𜵷𜵸𜵹𜵺𜵻𜵼𜵽𜵾𜵿𜶀𜶁𜶂𜶃𜶄𜶅𜶆𜶇𜶈𜶉𜶊𜶋𜶌𜶍𜶎𜶏▗𜶐𜶑𜶒𜶓▚𜶔𜶕𜶖𜶗▐𜶘𜶙𜶚𜶛▜𜶜𜶝𜶞𜶟𜶠𜶡𜶢𜶣𜶤𜶥𜶦𜶧𜶨𜶩𜶪𜶫"
        "▂𜶬𜶭𜶮𜶯𜶰𜶱𜶲𜶳𜶴𜶵𜶶𜶷𜶸𜶹𜶺𜶻𜶼𜶽𜶾𜶿𜷀𜷁𜷂𜷃𜷄𜷅𜷆𜷇𜷈𜷉𜷊𜷋𜷌𜷍𜷎𜷏𜷐𜷑𜷒𜷓𜷔𜷕𜷖𜷗𜷘𜷙𜷚▄𜷛𜷜𜷝𜷞▙𜷟𜷠𜷡𜷢▟𜷣▆𜷤𜷥█"
    )
    # partial infill pixels(pips) glyxel look up table, columns x rows
    pips_glut = [[], ["", "", ""], ["", "", "", "", ""]]
    pips_glut[1][1] = " ●"
    pips_glut[1][2] = " ᛫.:"
    pips_glut[2][2] = " 𜰡𜰢𜰣𜰤𜰥𜰦𜰧𜰨𜰩𜰪𜰫𜰬𜰭𜰮𜰯"
    pips_glut[2][3] = " 𜹑𜹒𜹓𜹔𜹕𜹖𜹗𜹘𜹙𜹚𜹛𜹜𜹝𜹞𜹟𜹠𜹡𜹢𜹣𜹤𜹥𜹦𜹧𜹨𜹩𜹪𜹫𜹬𜹭𜹮𜹯𜹰𜹱𜹲𜹳𜹴𜹵𜹶𜹷𜹸𜹹𜹺𜹻𜹼𜹽𜹾𜹿𜺀𜺁𜺂𜺃𜺄𜺅𜺆𜺇𜺈𜺉𜺊𜺋𜺌𜺍𜺎𜺏"
    pips_glut[2][4] = (
        "⠀⠁⠈⠉⠂⠃⠊⠋⠐⠑⠘⠙⠒⠓⠚⠛⠄⠅⠌⠍⠆⠇⠎⠏⠔⠕⠜⠝⠖⠗⠞⠟⠠⠡⠨⠩⠢⠣⠪⠫⠰⠱⠸⠹⠲⠳⠺⠻⠤⠥⠬⠭⠦⠧⠮⠯⠴⠵⠼⠽⠶⠷⠾⠿"
        "⡀⡁⡈⡉⡂⡃⡊⡋⡐⡑⡘⡙⡒⡓⡚⡛⡄⡅⡌⡍⡆⡇⡎⡏⡔⡕⡜⡝⡖⡗⡞⡟⡠⡡⡨⡩⡢⡣⡪⡫⡰⡱⡸⡹⡲⡳⡺⡻⡤⡥⡬⡭⡦⡧⡮⡯⡴⡵⡼⡽⡶⡷⡾⡿"
        "⢀⢁⢈⢉⢂⢃⢊⢋⢐⢑⢘⢙⢒⢓⢚⢛⢄⢅⢌⢍⢆⢇⢎⢏⢔⢕⢜⢝⢖⢗⢞⢟⢠⢡⢨⢩⢢⢣⢪⢫⢰⢱⢸⢹⢲⢳⢺⢻⢤⢥⢬⢭⢦⢧⢮⢯⢴⢵⢼⢽⢶⢷⢾⢿"
        "⣀⣁⣈⣉⣂⣃⣊⣋⣐⣑⣘⣙⣒⣓⣚⣛⣄⣅⣌⣍⣆⣇⣎⣏⣔⣕⣜⣝⣖⣗⣞⣟⣠⣡⣨⣩⣢⣣⣪⣫⣰⣱⣸⣹⣲⣳⣺⣻⣤⣥⣬⣭⣦⣧⣮⣯⣴⣵⣼⣽⣶⣷⣾⣿"
    )

    @staticmethod
    def image2slate(image, basis=(2, 4), pips=False):
        """A fast method to convert a PIL image into a slate (list of strips)"""
        _, _, x_size, y_size = image.getbbox()
        dx, dy = basis
        glut = ToGlyxels.pips_glut if pips else ToGlyxels.full_glut
        slate = []

        for y_pos in range(0, y_size, dy):
            y_strip = []
            for x_pos in range(0, x_size, dx):
                cell_img = image.crop((x_pos, y_pos, x_pos + dx, y_pos + dy))
                glut_idx, glyph_sty = ToGlyxels._img4cell2vals4seg(cell_img)
                glyph = glut[dx][dy][glut_idx]
                y_strip.append(Segment(glyph, glyph_sty))
            slate.append(Strip(y_strip))

        return slate

    @staticmethod
    def _img4cell2vals4seg(image):
        """Compute glyph look up table offset and associated style coloring"""
        fg = []
        bg = []
        glut_idx = 0

        duotone = image.quantize(colors=2)
        for idx, test_gx in enumerate(list(duotone.getdata())):
            if test_gx:
                fg.append(image.getdata()[idx])
                glut_idx += 2**idx
            else:
                bg.append(image.getdata()[idx])

        fg_color = ToGlyxels._colors2rgb4sty(fg)
        bg_color = ToGlyxels._colors2rgb4sty(bg)
        glyph_sty = Style.parse(" on ".join([fg_color, bg_color]))

        return (glut_idx, glyph_sty)

    @staticmethod
    def _colors2rgb4sty(rgb_list):
        """Compute broken but fast RGB centroid"""
        n = len(rgb_list)
        if n == 0:
            R = G = B = 0
        else:
            v_sum = [sum(x) for x in zip(*rgb_list)]
            R, G, B = [int(x / n) for x in v_sum]
        return f"rgb({R},{G},{B})"

    @staticmethod
    def pane2slate(pane, style: Style | None, basis, pips) -> List[List[Segment]]:
        """accept a PIL mask with dimensions (pane) and return a list of Textual strips"""
        x, y, mask = pane
        if x == 0 or y == 0:
            return [Strip.blank(0)]

        glut = ToGlyxels.pips_glut if pips else ToGlyxels.full_glut

        selection = Image.new("1", (x, y))
        selection.putdata(mask)
        # glyph based pixels must be an integer multiple of glyph cell basis, ie. 2x4 -> octants
        while x % basis[0] != 0:
            x += 1
        while y % basis[1] != 0:
            y += 1
        pane = Image.new("1", (x, y))
        # place bitmap into upper left corner
        pane.paste(selection, (0, 0))

        base_row = int(y / basis[1] - 1)
        mid_row = int(base_row / 2)
        cap_row = 0

        slate = []
        for y_glyph in range(0, y, basis[1]):
            y_strip = []
            y_row = int(y_glyph / basis[1])
            y_style = ToGlyxels._y_style(style, cap_row, mid_row, base_row, y_row)
            for x_glyph in range(0, x, basis[0]):
                glyph_idx = 0
                glyxel_list = []
                for y_idx in range(basis[1]):
                    for x_idx in range(basis[0]):
                        glyxel_list.append(
                            pane.getpixel((x_glyph + x_idx, y_glyph + y_idx))
                        )
                for exp, g_color in enumerate(glyxel_list):
                    if g_color > 0:
                        glyph_idx += 2**exp
                glyph = glut[basis[0]][basis[1]][glyph_idx]
                y_strip.append(Segment(glyph, y_style))
            slate.append(Strip(y_strip))
        return slate

    @staticmethod
    def style_slate(slate, style):
        """re-style content of strips"""
        base_row = len(slate)
        mid_row = int(base_row / 2)
        cap_row = 0
        new_slate = []
        for y_row, y_strip in enumerate(slate):
            y_style = ToGlyxels._y_style(style, cap_row, mid_row, base_row, y_row)
            new_slate.append(y_strip.apply_style(y_style))
        return new_slate

    @staticmethod
    def _y_style(style, cap_row, mid_row, base_row, y_row):
        if style:
            if style.overline and y_row != cap_row:
                style = style + Style(overline=False)
            if style.strike and y_row != mid_row:
                style = style + Style(strike=False)
            if y_row != base_row:
                if style.underline:
                    style = style + Style(underline=False)
                if style.underline2:
                    style = style + Style(underline2=False)
        return style

    @staticmethod
    def slate_join(strips, slate):
        if len(strips) == 0:
            return slate
        joint = []
        for idx, line in enumerate(strips):
            joint.append(Strip.join((line, slate[idx])).simplify())
        return joint



"""Boilerplate code for testing purposes"""

from textual.app import App, ComposeResult
from textual_englyph import EnGlyphText


class Test(App):
    """Test console markup styling the englyph text use case"""

    def compose(self) -> ComposeResult:
        yield EnGlyphText("Hello [red]Textual!", font_name="/tmp/BirchLeaf.ttf", font_size=32)


if __name__ == "__main__":
    Test().run(inline=True, inline_no_clear=True)



"""Boilerplate code for testing purposes"""

from textual.app import App, ComposeResult
from textual.widget import Widget
from textual.widgets import Label


class MyWidget(Widget):
    def __init__(self):
        super().__init__()
        self.my_string = str(self.rich_style)

    def compose(self) -> ComposeResult:
        yield Label(self.my_string)
        yield Label(str(self.rich_style))


class Test(App):
    def compose(self) -> ComposeResult:
        yield MyWidget()


if __name__ == "__main__":
    Test().run(inline=True, inline_no_clear=True)



"""Create large text output module for Textual with custom widget EnGlyph"""

from rich.console import RenderableType

from textual.strip import Strip
from textual.widget import Widget

class EnGlyph(Widget, inherit_bindings=False):
    """
    Textual widget to show a variety of large text outputs.

    Args:
        renderable: Rich renderable or string to display
        basis:tuple[(2,4)], the (x,y) partitions of cell glyph pixels (glyxel | gx)
        pips:bool[False], show glyxels in reduced density

        name:str, Standard Textual Widget argument
        id:str, Standard Textual Widget argument
        classes:str, Standard Textual Widget argument
        disabled:bool, Standard Textual Widget argument
    """

    DEFAULT_CSS = """
    EnGlyph {
        color: $text;
        height: auto;
        width: auto;
    }
    """

    _slate = _slate_cache = [Strip.blank(0)]
    _pane = None

    def __init__(self, renderable, *args, basis=(2, 4), pips=False, **kwargs):
        super().__init__(*args, **kwargs)
        self.basis = basis
        self.pips = pips
        self._predicate = self._preprocess(renderable)

    def on_mount(self):
        self._process()

    def get_content_width(self, container=None, viewport=None):
        return self._slate[0].cell_length

    def get_content_height(self, container=None, viewport=None, width=None):
        return len(self._slate)

    def __add__(self, rhs):
        """create the union of two EnGlyphed widgets"""
        # return self._union( self, rhs )
        pass

    __radd__ = __add__

    def __sub__(self, rhs):
        """create the difference of two EnGlyphed widgetslinux disables"""
        return self._difference(self, rhs)

    def __mul__(self, rhs):
        """create the intersection of two EnGlyphed widgets"""
        return self._intersection(self, rhs)

    def __div__(self, rhs):
        """create the intersection of two EnGlyphed widgets"""
        return self._disection(self, rhs)

    def _union(self, rhs):
        for idy, strip in enumerate(rhs._slate):
            for idx, seg in enumerate(strip):
                pass

    def _intersection(self, rhs):
        if isinstance(rhs, float):
            pass

    def __str__(self) -> str:
        output = [strip.text for strip in self._slate]
        return "\n".join(output)

    def _preprocess(self) -> None:
        """A stub handler for processing the input _predicate to the renderable"""
        pass

    def _process(self) -> None:
        """A stub handler for processing a renderable"""
        pass

    def _postprocess(self) -> None:
        """A stub handler to cache a slate (list of strips) for rendering"""
        pass

    def update(
        self,
        renderable: RenderableType | None = None,
        basis: tuple | None = None,
        pips: bool | None = None,
        font_size: int | None = None,
    ) -> None:
        """New display input"""
        self.basis = basis or self.basis
        self.pips = pips or self.pips
        self._font_size = font_size or self._font_size
        self._predicate = self._preprocess(renderable)
        self._process()
        self._postprocess()
        self.refresh(layout=True)

    def render_line(self, y: int) -> Strip:
        self._postprocess()
        strip = Strip.blank(0)
        if y < self.get_content_height():
            strip = self._slate[y]
        return strip



from textual_englyph import EnGlyphText
from textual.app import App, ComposeResult
from textual.widgets import Button, Label
from textual.containers import Vertical


class Test(App):
    fsize = 11

    def compose(self) -> ComposeResult:
        with Vertical():
            self.enhello = EnGlyphText("Hello Textual!", id="enhello")
            yield Label("A Button", id="enlabel")
            yield EnGlyphText("+")
            yield self.enhello
            yield EnGlyphText("=")
            yield Button(str(self.enhello), variant="primary")
            yield Label(str(self.enhello))

    def on_button_pressed(self):
        self.fsize += 1
        H = self.query_one("#enhello")
        H.update(font_size=self.fsize)
        self.query_one("#enlabel").update(str(H._font_size) + "gx")


if __name__ == "__main__":
    Test().run()



"""Connect local module file to module name"""

from ._englyph_image import EnGlyphImage
from ._englyph_text import EnGlyphText
from ._englyph_seven_segment import EnSevSeg

__all__ = ["EnGlyphImage", "EnGlyphText"]



"""Boilerplate code for testing purposes"""

from textual.app import App, ComposeResult
from textual.widgets import Label
from textual_englyph import EnGlyphText


class Test(App):
    """Test CSS and console markup styling the basic englyph use case"""

    DEFAULT_CSS = """
    EnGlyph {
        color: green;
        text-style: underline;
        }
    """

    def compose(self) -> ComposeResult:
        yield Label("Hello [blue]Textual!")
        yield EnGlyphText("Hello [blue]Textual!")


if __name__ == "__main__":
    Test().run(inline=True, inline_no_clear=True)
